const Partitioner = require('../src/partitioner.js')

const { performance } = require('perf_hooks')

describe('Testsuite partitioning', () => {
  it('should create empty partitions for empty test suites', async () => {
    const partitioner = new Partitioner({
      suiteFilePath: 'test/fixtures/empty.json'
    })
    expect(await partitioner.getPartition(0, 3)).to.deep.equal([])
  })

  it('should partition all tests for empty suites', async () => {
    const partitioner = new Partitioner({
      suiteFilePath: 'test/fixtures/empty.json'
    })
    partitioner.allTests = [
      { fullTitle () { return 'test 1' } },
      { fullTitle () { return 'test 2' } },
      { fullTitle () { return 'test 3' } }
    ]
    expect(await partitioner.getPartition(0, 3)).to.deep.equal(['test 3'])
    expect(await partitioner.getPartition(1, 3)).to.deep.equal(['test 1'])
    expect(await partitioner.getPartition(2, 3)).to.deep.equal(['test 2'])
  })

  it('should partition all tests with complete suites', async () => {
    const partitioner = new Partitioner({
      suiteFilePath: 'test/fixtures/complete.json'
    })
    partitioner.allTests = [
      { fullTitle () { return 'test 1' } },
      { fullTitle () { return 'test 2' } },
      { fullTitle () { return 'test 3' } }
    ]
    expect(await partitioner.getPartition(0, 3)).to.deep.equal(['test 1'])
    expect(await partitioner.getPartition(1, 3)).to.deep.equal(['test 2', 'test 3'])
    expect(await partitioner.getPartition(2, 3)).to.deep.equal([])
  })

  it('should not run tests in locked suites', async () => {
    const partitioner = new Partitioner({
      suiteFilePath: 'test/fixtures/locked.json'
    })
    partitioner.allTests = [
      { fullTitle () { return 'test 1' } },
      { fullTitle () { return 'test 2' } },
      { fullTitle () { return 'test 3' } }
    ]
    expect(await partitioner.getPartition(0, 3)).to.deep.equal(['test 1'])
    expect(await partitioner.getPartition(1, 3)).to.deep.equal(['test 2'])
    expect(await partitioner.getPartition(2, 3)).to.deep.equal([])
  })

  it('should run all tests for unlocked suites', async () => {
    const partitioner = new Partitioner({
      suiteFilePath: 'test/fixtures/unlocked.json'
    })
    partitioner.allTests = [
      { fullTitle () { return 'test 1' } },
      { fullTitle () { return 'test 2' } },
      { fullTitle () { return 'test 3' } }
    ]
    expect(await partitioner.getPartition(0, 3)).to.deep.equal(['test 1'])
    expect(await partitioner.getPartition(1, 3)).to.deep.equal(['test 2'])
    expect(await partitioner.getPartition(2, 3)).to.deep.equal(['test 3'])
  })

  it('should only consider tests in allTest array for partitioning', async () => {
    const partitioner = new Partitioner({
      suiteFilePath: 'test/fixtures/many_old_tests.json'
    })
    partitioner.allTests = [
      { fullTitle () { return 'test 1' } },
      { fullTitle () { return 'test 2' } },
      { fullTitle () { return 'test 3' } }
    ]
    expect(await partitioner.getPartition(0, 1)).to.deep.equal(['test 1', 'test 2', 'test 3'])
  })

  it('should only consider tests that match provided grep pattern', async () => {
    const partitioner = new Partitioner({ suiteFilePath: 'test/fixtures/complete.json' })
    partitioner.grep = /test 1/
    partitioner.allTests = [
      { fullTitle () { return 'test 1' } },
      { fullTitle () { return 'test 2' } },
      { fullTitle () { return 'test 3' } }
    ]
    expect(await partitioner.getPartition(0, 1)).to.deep.equal(['test 1'])
  })

  it('should be performant for a large list of tests', async () => {
    const partitioner = new Partitioner({ suiteFilePath: 'test/fixtures/complete.json' })
    partitioner.allTests = new Array(10000).fill(0).map((_, i) => ({
      fullTitle () { return `test ${i}` }
    }))
    const start = performance.now()
    await partitioner.getPartition(0, 100)
    const end = performance.now()
    const time = Math.floor(end - start)

    expect(time).to.be.below(500)
  })

  describe('get enabled tests', () => {
    const l1 = [
      { fullTitle: 'test 1', enabled: true },
      { fullTitle: 'test 4' }
    ]
    const l2 = [
      { fullTitle: () => 'test 1', enabled: true },
      { fullTitle: () => 'test 2', enabled: false },
      { fullTitle: () => 'test 3', enabled: 'some comment on why it is disabled' },
      { fullTitle: () => 'test 4' }
    ]
    it('should return list of enabled tests', () => {
      const partitioner = new Partitioner({
        suiteFilePath: 'test/fixtures/empty.json'
      })
      partitioner.suite = {
        tests: l2.map(t => {
          const test = { fullTitle: t.fullTitle() }
          if (Object.hasOwn(t, 'enabled')) test.enabled = t.enabled
          return test
        })
      }
      partitioner.allTests = l2

      expect(partitioner.enabledTests).to.deep.equal(l1)
    })
  })
})
